Attribute VB_Name = "modRotateText"
'*******************************************************************************************
'* Module to print (rotated, aligned, multiline) text in a picture box (angle mod 90 = 0): *
'*******************************************************************************************

Option Explicit

Const LF_FACESIZE = 32

Private Type LOGFONT

  lfHeight         As Long
  lfWidth          As Long
  lfEscapement     As Long
  lfOrientation    As Long
  lfWeight         As Long
  lfItalic         As Byte
  lfUnderline      As Byte
  lfStrikeOut      As Byte
  lfCharSet        As Byte
  lfOutPrecision   As Byte
  lfClipPrecision  As Byte
  lfQuality        As Byte
  lfPitchAndFamily As Byte
  lfFaceName       As String * LF_FACESIZE
  
End Type

Private Declare Function SetGraphicsMode Lib "gdi32" (ByVal hDC As Long, ByVal iMode As Long) As Long
Private Declare Function MulDiv Lib "kernel32" (ByVal nNumber As Long, ByVal nNumerator As Long, ByVal nDenominator As Long) As Long
Private Declare Function GetDeviceCaps Lib "gdi32" (ByVal hDC As Long, ByVal nIndex As Long) As Long
Private Declare Function CreateFontIndirect Lib "gdi32" Alias "CreateFontIndirectA" (lpLogFont As LOGFONT) As Long
Private Declare Function SelectObject Lib "gdi32" (ByVal hDC As Long, ByVal hObject As Long) As Long
Private Declare Function DeleteObject Lib "gdi32" (ByVal hObject As Long) As Long

Private Const RGN_OR = 2

Private Type RECT

  Left   As Long
  Top    As Long
  Right  As Long
  Bottom As Long
  
End Type

Private Declare Sub SetWindowRgn Lib "user32" (ByVal HWND&, ByVal hRgn&, ByVal bRedraw&)
Private Declare Function GetPixel& Lib "gdi32" (ByVal hDC&, ByVal X&, ByVal Y&)
Private Declare Function CreateRectRgn& Lib "gdi32" (ByVal X1&, ByVal Y1&, ByVal X2&, ByVal Y2&)
Private Declare Function CombineRgn& Lib "gdi32" (ByVal hDestRgn&, ByVal hSrcRgn1&, ByVal hSrcRgn2&, ByVal nMode&)
Private Declare Function GetClientRect& Lib "user32" (ByVal HWND As Long, lpRect As RECT)

Private iLeft   As Integer 'Coordinates (in pixels) to improve efficiency of MakeTransparentRegion
Private iTop    As Integer
Private iRight  As Integer
Private iBottom As Integer

'Print rotated text
'PB is a picture box.
'Text is the string to be printed (possibly multiline).
'Angle is the orientation, in degrees.
'Align is the alignment (LEFT/CENTER/RIGHT).
'Note: you get best results when using TrueType fonts.
Public Sub PrintRotatedText(ByRef PB As PictureBox, ByVal Text As String, ByVal Angle As Integer, ByVal Align As String)

  Dim hFont As Long, holdfont As Long
  Dim Font As LOGFONT
  
  Const GM_ADVANCED = 2
  Const LOGPIXELSY = 90
  
  If Angle Mod 90 <> 0 Then Exit Sub
  If Align = "JUSTIFY" Then Align = "LEFT"
  
  SetGraphicsMode PB.hDC, GM_ADVANCED
  
  'Create a Font object, similar to the current font in PB but with a different orientation:
  Font.lfHeight = -MulDiv(PB.fontSize, GetDeviceCaps(PB.hDC, LOGPIXELSY), 72)
  Font.lfWidth = 0
  Font.lfEscapement = Angle * 10
  Font.lfOrientation = Angle * 10
  Font.lfWeight = IIf(PB.FontBold, 700, 400)
  Font.lfItalic = IIf(PB.FontItalic, 1, 0)
  Font.lfUnderline = IIf(PB.FontUnderline, 1, 0)
  Font.lfStrikeOut = IIf(PB.FontStrikethru, 1, 0)
  Font.lfCharSet = 0
  Font.lfOutPrecision = 0
  Font.lfClipPrecision = 0
  Font.lfQuality = 2
  Font.lfPitchAndFamily = 33
  Font.lfFaceName = PB.FontName & vbNullChar
  
  hFont = CreateFontIndirect(Font)
  holdfont = SelectObject(PB.hDC, hFont)
  
  iLeft = PB.ScaleWidth
  iTop = PB.ScaleHeight
  iRight = 0
  iBottom = 0
  
  'Do the printing:
  
  Dim sCurText  As String
  Dim iOffset   As Integer
  Dim iFullLine As Integer 'Full line length in pixels
  
  If Angle Mod 180 = 0 Then
  
    iFullLine = PB.ScaleWidth
    
  Else
  
    iFullLine = PB.ScaleHeight
    
  End If
  
  Do
  
    If InStr(Text, vbCrLf) Then
    
      sCurText = Left(Text, InStr(Text, vbCrLf) - 1)
      
      If PB.TextWidth(sCurText) > iFullLine Then
      
        GoTo FullLine
        
      Else
      
        Text = Right(Text, Len(Text) - Len(sCurText) - 2)
        SetStartPosition PB, sCurText, Angle, Align, iOffset
        PB.Print sCurText
        iOffset = iOffset + PB.TextHeight(sCurText)
        
      End If
      
    ElseIf PB.TextWidth(Text) > iFullLine Then
    
FullLine:
      If InStr(Text, " ") Then
      
        sCurText = vbNullString
        
        Do
        
          If InStr(Text, " ") Then
          
            If sCurText = vbNullString Then
            
              sCurText = Left(Text, InStr(Text, " ") - 1)
              Text = Right(Text, Len(Text) - Len(sCurText) - 1)
              
            ElseIf PB.TextWidth(sCurText & " " & Left(Text, InStr(Text, " ") - 1)) <= iFullLine Then
            
              Dim sThisText As String
              
              sThisText = Left(Text, InStr(Text, " ") - 1)
              sCurText = sCurText & " " & sThisText
              Text = Right(Text, Len(Text) - Len(sThisText) - 1)
              
            Else
            
              SetStartPosition PB, sCurText, Angle, Align, iOffset
              PB.Print sCurText
              iOffset = iOffset + PB.TextHeight(sCurText)
              Exit Do
              
            End If
            
          Else
          
            SetStartPosition PB, sCurText, Angle, Align, iOffset
            PB.Print sCurText
            iOffset = iOffset + PB.TextHeight(sCurText)
            Exit Do
            
          End If
          
        Loop
        
      Else
      
        SetStartPosition PB, Text, Angle, Align, iOffset
        PB.Print Text
        Exit Do
        
      End If
      
    Else
    
      SetStartPosition PB, Text, Angle, Align, iOffset
      PB.Print Text
      Exit Do
      
    End If
    
  Loop
  
  'Reselect the old font:
  SelectObject PB.hDC, holdfont
  
  'Destroy the font object just created:
  DeleteObject hFont
  
  If Text <> vbNullString Then
  
    Dim hRegion As Long
    
    With PB
    
      .ScaleMode = vbPixels
      .BorderStyle = 0
      DoEvents 'Let the picture box paint itself
      hRegion = MakeTransparentRegion(PB)
      SetWindowRgn .HWND, hRegion, 1
      DeleteObject hRegion
      
    End With
    
  End If
  
End Sub

'Set start position of this single line Text
Private Sub SetStartPosition(ByRef oPB As PictureBox, ByVal Text As String, ByVal Angle As Integer, ByVal Align As String, ByVal Offset As Integer)

  Dim iCurLeft   As Integer
  Dim iCurTop    As Integer
  Dim iCurRight  As Integer
  Dim iCurBottom As Integer
  
  oPB.CurrentX = 0
  oPB.CurrentY = 0
  
  If Angle = 0 Then
  
    If Align = "LEFT" Then
    
      oPB.CurrentY = Offset
      
    ElseIf Align = "CENTER" Then
    
      oPB.CurrentX = oPB.ScaleWidth / 2 - oPB.TextWidth(Text) / 2
      oPB.CurrentY = Offset
      
    ElseIf Align = "RIGHT" Then
    
      oPB.CurrentX = oPB.ScaleWidth - oPB.TextWidth(Text)
      oPB.CurrentY = Offset
      
    End If
    
    iCurLeft = oPB.CurrentX
    iCurTop = 0
    iCurRight = oPB.CurrentX + oPB.TextWidth(Text)
    iCurBottom = oPB.CurrentY + oPB.TextHeight(Text)
    
  ElseIf Angle = -90 Or Angle = 270 Then
  
    If Align = "LEFT" Then
    
      oPB.CurrentX = oPB.ScaleWidth - Offset
      
    ElseIf Align = "CENTER" Then
    
      oPB.CurrentX = oPB.ScaleWidth - Offset
      oPB.CurrentY = oPB.ScaleHeight / 2 - oPB.TextWidth(Text) / 2
      
    ElseIf Align = "RIGHT" Then
    
      oPB.CurrentX = oPB.ScaleWidth - Offset
      oPB.CurrentY = oPB.ScaleHeight - oPB.TextWidth(Text)
      
    End If
    
    iCurLeft = oPB.CurrentX - oPB.TextHeight(Text)
    iCurTop = oPB.CurrentY
    iCurRight = oPB.ScaleWidth
    iCurBottom = oPB.CurrentY + oPB.TextWidth(Text)
    
  ElseIf Angle = -180 Or Angle = 180 Then
  
    If Align = "LEFT" Then
    
      oPB.CurrentX = oPB.ScaleWidth
      oPB.CurrentY = oPB.ScaleHeight - Offset
      
    ElseIf Align = "CENTER" Then
    
      oPB.CurrentX = oPB.ScaleWidth / 2 + oPB.TextWidth(Text) / 2
      oPB.CurrentY = oPB.ScaleHeight - Offset
      
    ElseIf Align = "RIGHT" Then
    
      oPB.CurrentX = oPB.TextWidth(Text)
      oPB.CurrentY = oPB.ScaleHeight - Offset
      
    End If
    
    iCurLeft = oPB.CurrentX - oPB.TextWidth(Text)
    iCurTop = oPB.CurrentY - oPB.TextHeight(Text)
    iCurRight = oPB.CurrentX
    iCurBottom = oPB.ScaleHeight
    
  ElseIf Angle = -270 Or Angle = 90 Then
  
    If Align = "LEFT" Then
    
      oPB.CurrentX = Offset
      oPB.CurrentY = oPB.ScaleHeight
      
    ElseIf Align = "CENTER" Then
    
      oPB.CurrentX = Offset
      oPB.CurrentY = oPB.ScaleHeight / 2 + oPB.TextWidth(Text) / 2
      
    ElseIf Align = "RIGHT" Then
    
      oPB.CurrentX = Offset
      oPB.CurrentY = oPB.TextWidth(Text)
      
    End If
    
    iCurLeft = 0
    iCurTop = oPB.CurrentY - oPB.TextWidth(Text)
    iCurRight = oPB.CurrentX + oPB.TextHeight(Text)
    iCurBottom = oPB.CurrentY
    
  End If
  
  If iCurLeft < iLeft Then iLeft = iCurLeft
  If iCurTop < iTop Then iTop = iCurTop
  If iCurRight > iRight Then iRight = iCurRight
  If iCurBottom > iBottom Then iBottom = iCurBottom
  
End Sub

'Make region transparent
Private Function MakeTransparentRegion(ByRef oPictureBox As PictureBox) As Long

  Dim MaskColor&, X&, Y&, startX&, FullRegion&, LineRegion&
  Dim Inline As Boolean
  
  'Get the color of 0, 0 to be the transparent color:
  MaskColor = GetPixel(oPictureBox.hDC, 0, 0)
  
  If iLeft > 0 Then iLeft = iLeft - 1
  If iTop > 0 Then iTop = iTop - 1
  If iRight < oPictureBox.ScaleWidth Then iRight = iRight + 1
  If iBottom < oPictureBox.ScaleHeight Then iBottom = iBottom + 1
  
  For Y = iTop To iBottom
  
    For X = iLeft To iRight
    
      If (GetPixel(oPictureBox.hDC, X, Y) = MaskColor) Or (X = oPictureBox.ScaleWidth) Then
      
        If Inline = True Then
        
          Inline = False
          LineRegion = CreateRectRgn(startX, Y, X, Y + 1)
          
          If FullRegion = 0 Then
          
            FullRegion = LineRegion
            
          Else
          
            CombineRgn FullRegion, FullRegion, LineRegion, RGN_OR
            DeleteObject LineRegion
            
          End If
          
        End If
        
      ElseIf Inline = False Then
      
        Inline = True
        startX = X
        
      End If
      
    Next X
    
  Next Y
  
  MakeTransparentRegion = FullRegion
  
End Function
